#!/usr/bin/env python3
"""
Windows Disk Cleanup Utility (System Management Tool)

Parameters:
    --analyze DRIVE: Analyze disk for cleanup
    --cleanup DRIVE: Perform disk cleanup
    --temp: Clean temporary files
    --recycle: Empty recycle bin
    --windows: Clean Windows components
    --apps: Clean application data
    --format FORMAT: Output format (table/json)

Requires: win32api, shutil, tabulate packages and admin privileges
Warning: Cleaned files cannot be recovered - verify selections
"""

import os
import sys
import json
import time
import shutil
import logging
import tempfile
import subprocess
from typing import List, Dict, Optional, Union, Tuple
from datetime import datetime
from pathlib import Path

import win32com.client
import win32api
import win32con
import win32file
from tabulate import tabulate

# Configure logging
logging.basicConfig(
    level=logging.INFO,
    format='%(asctime)s - %(levelname)s - %(message)s'
)
logger = logging.getLogger(__name__)

class DiskCleanupUtility:
    """Manages disk cleanup operations."""
    
    def __init__(self):
        """Initialize the DiskCleanupUtility."""
        self.temp_paths = [
            os.environ.get("TEMP"),
            os.environ.get("TMP"),
            os.path.join(os.environ.get("WINDIR", "C:\\Windows"), "Temp")
        ]
        self.browser_paths = {
            "chrome": os.path.join(
                os.environ.get("LOCALAPPDATA", ""),
                "Google\\Chrome\\User Data\\Default\\Cache"
            ),
            "firefox": os.path.join(
                os.environ.get("LOCALAPPDATA", ""),
                "Mozilla\\Firefox\\Profiles"
            ),
            "edge": os.path.join(
                os.environ.get("LOCALAPPDATA", ""),
                "Microsoft\\Edge\\User Data\\Default\\Cache"
            )
        }
        self.windows_update_path = os.path.join(
            os.environ.get("WINDIR", "C:\\Windows"),
            "SoftwareDistribution\\Download"
        )
        self.recycle_bin_paths = [
            drive + ":\\$Recycle.Bin"
            for drive in self.get_drive_letters()
        ]
        self.downloads_path = os.path.join(
            os.path.expanduser("~"),
            "Downloads"
        )
    
    def get_drive_letters(self) -> List[str]:
        """
        Get list of drive letters.
        
        Returns:
            List[str]: List of drive letters
        """
        try:
            drives = []
            bitmask = win32api.GetLogicalDrives()
            for letter in "ABCDEFGHIJKLMNOPQRSTUVWXYZ":
                if bitmask & 1:
                    drives.append(letter)
                bitmask >>= 1
            return drives
            
        except Exception as e:
            logger.error(f"Error getting drive letters: {e}")
            return []
    
    def get_directory_size(self, path: str) -> int:
        """
        Calculate total size of a directory.
        
        Args:
            path: Directory path
            
        Returns:
            int: Total size in bytes
        """
        total_size = 0
        try:
            for dirpath, dirnames, filenames in os.walk(path):
                for filename in filenames:
                    file_path = os.path.join(dirpath, filename)
                    try:
                        total_size += os.path.getsize(file_path)
                    except (OSError, IOError):
                        continue
        except (OSError, IOError):
            pass
        return total_size
    
    def clean_directory(self, path: str, min_age: Optional[int] = None) -> Tuple[int, int]:
        """
        Clean a directory by removing files and subdirectories.
        
        Args:
            path: Directory path
            min_age: Minimum age in days for files to be deleted
            
        Returns:
            Tuple[int, int]: (bytes cleaned, files cleaned)
        """
        if not os.path.exists(path):
            return 0, 0
            
        bytes_cleaned = 0
        files_cleaned = 0
        current_time = time.time()
        
        try:
            for root, dirs, files in os.walk(path, topdown=False):
                # Remove files
                for name in files:
                    file_path = os.path.join(root, name)
                    try:
                        if min_age is not None:
                            # Check file age
                            file_time = os.path.getmtime(file_path)
                            if (current_time - file_time) < (min_age * 86400):
                                continue
                                
                        size = os.path.getsize(file_path)
                        os.remove(file_path)
                        bytes_cleaned += size
                        files_cleaned += 1
                        
                    except (OSError, IOError):
                        continue
                
                # Remove empty directories
                for name in dirs:
                    dir_path = os.path.join(root, name)
                    try:
                        os.rmdir(dir_path)
                    except (OSError, IOError):
                        continue
                        
        except Exception as e:
            logger.error(f"Error cleaning directory {path}: {e}")
            
        return bytes_cleaned, files_cleaned
    
    def clean_temp_files(self, min_age: Optional[int] = None) -> Dict[str, Tuple[int, int]]:
        """
        Clean temporary files.
        
        Args:
            min_age: Minimum age in days for files to be deleted
            
        Returns:
            Dict[str, Tuple[int, int]]: Results for each temp directory
        """
        results = {}
        
        for path in self.temp_paths:
            if path and os.path.exists(path):
                results[path] = self.clean_directory(path, min_age)
                
        return results
    
    def clean_browser_caches(self, browsers: Optional[List[str]] = None,
                           min_age: Optional[int] = None) -> Dict[str, Tuple[int, int]]:
        """
        Clean browser caches.
        
        Args:
            browsers: List of browsers to clean ("chrome", "firefox", "edge")
            min_age: Minimum age in days for files to be deleted
            
        Returns:
            Dict[str, Tuple[int, int]]: Results for each browser
        """
        results = {}
        
        if browsers is None:
            browsers = list(self.browser_paths.keys())
            
        for browser in browsers:
            if browser in self.browser_paths:
                path = self.browser_paths[browser]
                if os.path.exists(path):
                    results[browser] = self.clean_directory(path, min_age)
                    
        return results
    
    def clean_windows_update(self) -> Tuple[int, int]:
        """
        Clean Windows Update files.
        
        Returns:
            Tuple[int, int]: (bytes cleaned, files cleaned)
        """
        if os.path.exists(self.windows_update_path):
            return self.clean_directory(self.windows_update_path)
        return 0, 0
    
    def clean_recycle_bin(self) -> Dict[str, Tuple[int, int]]:
        """
        Clean Recycle Bin.
        
        Returns:
            Dict[str, Tuple[int, int]]: Results for each drive
        """
        results = {}
        
        for path in self.recycle_bin_paths:
            if os.path.exists(path):
                results[path] = self.clean_directory(path)
                
        return results
    
    def clean_downloads(self, min_age: Optional[int] = None) -> Tuple[int, int]:
        """
        Clean Downloads folder.
        
        Args:
            min_age: Minimum age in days for files to be deleted
            
        Returns:
            Tuple[int, int]: (bytes cleaned, files cleaned)
        """
        if os.path.exists(self.downloads_path):
            return self.clean_directory(self.downloads_path, min_age)
        return 0, 0
    
    def clean_all(self, min_age: Optional[int] = None) -> Dict[str, Dict]:
        """
        Run all cleanup operations.
        
        Args:
            min_age: Minimum age in days for files to be deleted
            
        Returns:
            Dict[str, Dict]: Results for all operations
        """
        results = {
            "temp_files": self.clean_temp_files(min_age),
            "browser_caches": self.clean_browser_caches(min_age=min_age),
            "windows_update": {"update": self.clean_windows_update()},
            "recycle_bin": self.clean_recycle_bin(),
            "downloads": {"downloads": self.clean_downloads(min_age)}
        }
        
        # Calculate totals
        total_bytes = 0
        total_files = 0
        
        for category in results.values():
            for result in category.values():
                total_bytes += result[0]
                total_files += result[1]
                
        results["total"] = {
            "bytes_cleaned": total_bytes,
            "files_cleaned": total_files
        }
        
        return results

def format_size(size: int) -> str:
    """
    Format size in bytes to human-readable format.
    
    Args:
        size: Size in bytes
        
    Returns:
        str: Formatted size
    """
    for unit in ["B", "KB", "MB", "GB", "TB"]:
        if size < 1024:
            return f"{size:.2f} {unit}"
        size /= 1024
    return f"{size:.2f} PB"

def format_cleanup_results(results: Dict, format_type: str = "table") -> str:
    """
    Format cleanup results for output.
    
    Args:
        results: Cleanup results
        format_type: Output format ("table" or "json")
        
    Returns:
        str: Formatted results
    """
    if format_type == "json":
        return json.dumps(results, indent=2)
    
    # Format detailed results
    sections = []
    
    # Temporary Files
    if results.get("temp_files"):
        temp_data = [
            [path, format_size(bytes_), files]
            for path, (bytes_, files) in results["temp_files"].items()
        ]
        sections.append("\nTemporary Files:")
        sections.append(tabulate(
            temp_data,
            headers=["Path", "Space Cleaned", "Files Cleaned"],
            tablefmt="grid"
        ))
    
    # Browser Caches
    if results.get("browser_caches"):
        browser_data = [
            [browser.title(), format_size(bytes_), files]
            for browser, (bytes_, files) in results["browser_caches"].items()
        ]
        sections.append("\nBrowser Caches:")
        sections.append(tabulate(
            browser_data,
            headers=["Browser", "Space Cleaned", "Files Cleaned"],
            tablefmt="grid"
        ))
    
    # Windows Update
    if results.get("windows_update"):
        update_bytes, update_files = results["windows_update"]["update"]
        sections.append("\nWindows Update:")
        sections.append(tabulate(
            [[format_size(update_bytes), update_files]],
            headers=["Space Cleaned", "Files Cleaned"],
            tablefmt="grid"
        ))
    
    # Recycle Bin
    if results.get("recycle_bin"):
        recycle_data = [
            [path, format_size(bytes_), files]
            for path, (bytes_, files) in results["recycle_bin"].items()
        ]
        sections.append("\nRecycle Bin:")
        sections.append(tabulate(
            recycle_data,
            headers=["Drive", "Space Cleaned", "Files Cleaned"],
            tablefmt="grid"
        ))
    
    # Downloads
    if results.get("downloads"):
        download_bytes, download_files = results["downloads"]["downloads"]
        sections.append("\nDownloads:")
        sections.append(tabulate(
            [[format_size(download_bytes), download_files]],
            headers=["Space Cleaned", "Files Cleaned"],
            tablefmt="grid"
        ))
    
    # Total
    if results.get("total"):
        sections.append("\nTotal Cleanup:")
        sections.append(tabulate(
            [[
                format_size(results["total"]["bytes_cleaned"]),
                results["total"]["files_cleaned"]
            ]],
            headers=["Total Space Cleaned", "Total Files Cleaned"],
            tablefmt="grid"
        ))
    
    return "\n".join(sections)

def main():
    """Main function to handle command-line interface."""
    import argparse
    
    parser = argparse.ArgumentParser(description="Windows Disk Cleanup Utility")
    parser.add_argument("--temp", action="store_true",
                      help="Clean temporary files")
    parser.add_argument("--browser", action="store_true",
                      help="Clean browser caches")
    parser.add_argument("--browsers", nargs="+",
                      choices=["chrome", "firefox", "edge"],
                      help="Specific browsers to clean")
    parser.add_argument("--update", action="store_true",
                      help="Clean Windows Update files")
    parser.add_argument("--recycle", action="store_true",
                      help="Clean Recycle Bin")
    parser.add_argument("--downloads", action="store_true",
                      help="Clean Downloads folder")
    parser.add_argument("--all", action="store_true",
                      help="Run all cleanup operations")
    parser.add_argument("--min-age", type=int,
                      help="Minimum age in days for files to be deleted")
    parser.add_argument("--format", choices=["table", "json"],
                      default="table",
                      help="Output format (table or json)")
    
    args = parser.parse_args()
    
    cleanup_utility = DiskCleanupUtility()
    results = {}
    
    try:
        if args.all:
            results = cleanup_utility.clean_all(args.min_age)
            
        else:
            if args.temp:
                results["temp_files"] = cleanup_utility.clean_temp_files(args.min_age)
                
            if args.browser:
                results["browser_caches"] = cleanup_utility.clean_browser_caches(
                    args.browsers, args.min_age
                )
                
            if args.update:
                results["windows_update"] = {
                    "update": cleanup_utility.clean_windows_update()
                }
                
            if args.recycle:
                results["recycle_bin"] = cleanup_utility.clean_recycle_bin()
                
            if args.downloads:
                results["downloads"] = {
                    "downloads": cleanup_utility.clean_downloads(args.min_age)
                }
            
            if results:
                # Calculate totals
                total_bytes = 0
                total_files = 0
                
                for category in results.values():
                    for result in category.values():
                        total_bytes += result[0]
                        total_files += result[1]
                        
                results["total"] = {
                    "bytes_cleaned": total_bytes,
                    "files_cleaned": total_files
                }
        
        if results:
            print(format_cleanup_results(results, args.format))
        else:
            parser.print_help()
            
    except Exception as e:
        logger.error(f"An error occurred: {e}")
        sys.exit(1)

if __name__ == "__main__":
    main()
